title: Atomic Design
url: http://bradfrost.com/blog/post/atomic-web-design/
hash_url: 071269cdcaaf3e7bc70a4780b64662e6

<blockquote><p>We’re not designing pages, we’re designing systems of components.—<a href="http://bradfrost.com/blog/mobile/bdconf-stephen-hay-presents-responsive-design-workflow/">Stephen Hay</a></p></blockquote>
<p>As the craft of Web design continues to evolve, we’re recognizing the need to develop thoughtful design systems, rather than creating simple collections of web pages.</p>
<p>A lot has been said about creating <a title="http://24ways.org/2012/design-systems/" href="http://24ways.org/2012/design-systems/">design systems</a>, and much of it focuses on establishing foundations for color, typography, grids, texture and the like. This type of thinking is certainly important, but I’m slightly less interested in these aspects of design because ultimately they are and will always be subjective. Lately I’ve been more interested in what our interfaces are comprised of and how we can construct design systems in a more methodical way.</p>
<p>In searching for inspiration and parallels, I kept coming back to chemistry. The thought is that all matter (whether solid, liquid, gas, simple, complex, etc) is comprised of atoms. Those atomic units bond together to form molecules, which in turn combine into more complex organisms to ultimately create all matter in our universe.</p>
<p>Similarly, interfaces are made up of smaller components. This means we can break entire interfaces down into fundamental building blocks and work up from there. That’s the basic gist of atomic design.</p>

<h2>What is Atomic Design</h2>
<p>Atomic design is methodology for creating design systems. There are five distinct levels in atomic design:</p>
<ol>
<li><a href="#atoms">Atoms</a></li>
<li><a href="#molecules">Molecules</a></li>
<li><a href="#organisms">Organisms</a></li>
<li><a href="#templates">Templates </a></li>
<li><a href="#pages">Pages </a></li>
</ol>
<p><img class="alignnone size-large wp-image-7254" src="http://bradfrost.com/wp-content/uploads/2013/06/atomic-design.png" alt="The progression of atomic design: atoms to molecules to organiams to templates to pages"/></p>
<p>Let’s explore each stage in more detail.</p>
<h3 id="atoms">Atoms</h3>
<p>Atoms are the basic building blocks of matter. Applied to web interfaces, atoms are our HTML tags, such as a form label, an input or a button.</p>
<p><img class="aligncenter size-full wp-image-5992" src="http://bradfrost.com/wp-content/uploads/2013/06/atoms.jpg" alt="Atoms"/></p>
<p>Atoms can also include more abstract elements like color palettes, fonts and even more invisible aspects of an interface like animations.</p>
<p>Like atoms in nature they’re fairly abstract and often not terribly useful on their own. However, they’re good as a reference in the context of a pattern library as you can see all your global styles laid out at a glance.</p>
<h3 id="molecules">Molecules</h3>
<p>Things start getting more interesting and tangible when we start combining atoms together. Molecules are groups of atoms bonded together and are the smallest fundamental units of a compound. These molecules take on their own properties and serve as the backbone of our design systems.</p>
<p>For example, a form label, input or button aren’t too useful by themselves, but combine them together as a form and now they can actually do something together.</p>
<p><img class="aligncenter size-full wp-image-5993" src="http://bradfrost.com/wp-content/uploads/2013/06/molecule.jpg" alt="molecule"/></p>
<p>Building up to molecules from atoms encourages a “do one thing and do it well” mentality. While molecules can be complex, as a rule of thumb they are relatively simple combinations of atoms built for reuse.</p>
<h3 id="organisms">Organisms</h3>
<p>Molecules give us some building blocks to work with, and we can now combine them together to form organisms. Organisms are groups of molecules joined together to form a relatively complex, distinct section of an interface.</p>
<p><img class="aligncenter size-large wp-image-6002" src="http://bradfrost.com/wp-content/uploads/2013/06/organism2.jpg" alt="organism"/></p>
<p><img class="aligncenter size-large wp-image-6001" src="http://bradfrost.com/wp-content/uploads/2013/06/organism-examples.jpg" alt="organism-examples"/></p>
<p>We’re starting to get increasingly concrete. A client might not be terribly interested in the molecules of a design system, but with organisms we can see the final interface beginning to take shape. Dan Mall (who I’m working with on several projects) uses <a title="http://danielmall.com/articles/rif-element-collages/" href="http://danielmall.com/articles/rif-element-collages/">element collages</a>, which articulate ideas for a few key organisms to facilitate client conversations and shape the visual direction (all without having to construct full comps).</p>
<p>Organisms can consist of similar and/or different molecule types. For example, a masthead organism might consist of diverse components like a logo, primary navigation, search form, and list of social media channels. But a “product grid” organism might consist of the same molecule (possibly containing a product image, product title and price) repeated over and over again.</p>
<p>Building up from molecules to organisms encourages creating standalone, portable, reusable components.</p>
<h3 id="templates">Templates</h3>
<p>At the template stage, we break our chemistry analogy to get into language that makes more sense to our clients and our final output. Templates consist mostly of groups of organisms stitched together to form pages. It’s here where we start to see the design coming together and start seeing things like layout in action.</p>
<p><img class="aligncenter size-large wp-image-6000" src="http://bradfrost.com/wp-content/uploads/2013/06/template1.jpg" alt="template"/></p>
<p>Templates are very concrete and provide context to all these relatively abstract molecules and organisms. Templates are also where clients start seeing the final design in place. In my experience working with this methodology, templates begin their life as HTML wireframes, but over time increase fidelity to ultimately become the final deliverable. Bearded Studio in Pittsburgh follow <a title="http://alistapart.com/article/responsive-comping-obtaining-signoff-with-mockups" href="http://alistapart.com/article/responsive-comping-obtaining-signoff-with-mockups">a similar process</a>, where designs start <a title="http://aafh-css.heroku.com/wireframes-no-mq" href="http://aafh-css.heroku.com/wireframes-no-mq">grayscale and layout-less</a> but <a title="http://aafh-css.heroku.com/wireframes" href="http://aafh-css.heroku.com/wireframes">slowly</a> <a title="http://aafh-css.heroku.com/v1" href="http://aafh-css.heroku.com/v1">increase</a> <a title="http://aafh-css.heroku.com/v2" href="http://aafh-css.heroku.com/v2">fidelity</a> until the <a title="http://aafh-css.heroku.com/v5" href="http://aafh-css.heroku.com/v5">final design</a> is in place.</p>
<h3 id="pages">Pages</h3>
<p>Pages are specific instances of templates. Here, placeholder content is replaced with real representative content to give an accurate depiction of what a user will ultimately see.</p>
<p><img class="aligncenter size-large wp-image-5999" src="http://bradfrost.com/wp-content/uploads/2013/06/page1.jpg" alt="page"/></p>
<p>Pages are the highest level of fidelity and because they’re the most tangible, it’s typically where most people in the process spend most of their time and what most reviews revolve around.</p>
<p>The page stage is essential as it’s where we test the effectiveness of the design system. Viewing everything in context allows us to loop back to modify our molecules, organisms, and templates to better address the real context of the design.</p>
<p>Pages are also the place to test variations in templates. For example, you might want to articulate what a headline containing 40 characters looks like, but also demonstrate what 340 characters looks like. What does it look like when a user has one item in their shopping cart versus 10 items with a discount code applied? Again, these specific instances influence how we loop back through and construct our system.</p>
<h2>Why Atomic Design</h2>
<p>In a lot of ways, this is how we’ve been doing things all along, even if we haven’t been consciously thinking about it in this specific way.</p>
<p>Atomic design provides a clear methodology for crafting design systems. Clients and team members are able to better appreciate the concept of design systems by actually seeing the steps laid out in front of them.</p>
<p>Atomic design gives us the ability to traverse from abstract to concrete. Because of this, we can create systems that promote consistency and scalability while simultaneously showing things in their final context. And by assembling rather than deconstructing, we’re crafting a system right out of the gate instead of cherry picking patterns after the fact.</p>
<h2>Pattern Lab</h2>
<p>In order to apply this methodology in my work, I (along with the help of the great <a title="http://dmolsen.com/" href="http://dmolsen.com/">Dave Olsen</a>) created a tool called <a title="http://patternlab.bradfrostweb.com" href="http://patternlab.bradfrostweb.com">Pattern Lab</a> to actually create these atomic design systems. I’ll cover Pattern Lab in detail later, but feel free to <a title="https://github.com/bradfrost/patternlab" href="https://github.com/bradfrost/patternlab">check it out on Github</a>.</p>
<h2>Further Reading</h2>
<ul>
<li>So Andy Clarke has been setting the stage for these types of conversations for a long while now. In fact, he wrote a chapter for <em><a title="https://shop.smashingmagazine.com/smashing-book-3.html" href="https://shop.smashingmagazine.com/smashing-book-3.html">Smashing Book 3</a></em> called “Becoming Fabulously Flexible: Designing Atoms and Elements.” I had no idea that existed, so how about that! I highly encourage you to check that out. I also highly encourage you to take a look at his tool called <a title="http://malarkey.github.io/Rock-Hammer/" href="http://malarkey.github.io/Rock-Hammer/">Rock Hammer</a>, which is a great way to <a title="http://stuffandnonsense.co.uk/blog/about/rock-hammer-a-curated-responsive-project-library" href="http://stuffandnonsense.co.uk/blog/about/rock-hammer-a-curated-responsive-project-library">construct a pattern library</a> utilizing many of these principles.</li>
<li><a title="http://www.youtube.com/watch?v=fqULJBBEVQE" href="http://www.youtube.com/watch?v=fqULJBBEVQE">Web Components: A Tectonic Shift for Web Development</a> – Web Components seem to dovetail really nicely into the concept of atomic design, and watching this video will show why web components</li>
<li><a title="W3C Modularity" href="http://www.w3.org/DesignIssues/Modularity.html">Modularity</a> Tim Berners-Lee discusses how modularity as an important design principle for the Web.</li>
<li><a title="http://daverupert.com/2013/04/responsive-deliverables/" href="http://daverupert.com/2013/04/responsive-deliverables/">Responsive Deliverables</a> by Dave Rupert talks about the idea of constructing “Tiny Bootstraps, for Every Client”</li>
</ul>
